"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Babel = require("@babel/core");
const path_1 = require("path");
const source_map_support_1 = require("source-map-support");
const AllSyntaxPlugin_1 = require("./AllSyntaxPlugin");
const BabelPrinterPlugin_1 = require("./BabelPrinterPlugin");
const extensions_1 = require("./extensions");
const PluginLoader_1 = require("./PluginLoader");
const PrettierPrinterPlugin_1 = require("./PrettierPrinterPlugin");
const RecastPlugin_1 = require("./RecastPlugin");
const AstExplorerResolver_1 = require("./resolvers/AstExplorerResolver");
const FileSystemResolver_1 = require("./resolvers/FileSystemResolver");
const NetworkResolver_1 = require("./resolvers/NetworkResolver");
const PackageResolver_1 = require("./resolvers/PackageResolver");
const transpile_requires_1 = require("./transpile-requires");
class Plugin {
    constructor(rawPlugin, inferredName, source, resolvedPath) {
        this.rawPlugin = rawPlugin;
        this.inferredName = inferredName;
        this.source = source;
        this.resolvedPath = resolvedPath;
        let instance = rawPlugin(Babel);
        if (instance.name) {
            this.declaredName = instance.name;
        }
    }
}
exports.Plugin = Plugin;
function ignoreDotfiles(path, basename, root) {
    return basename.startsWith('.');
}
var Printer;
(function (Printer) {
    Printer["Recast"] = "recast";
    Printer["Prettier"] = "prettier";
    Printer["Babel"] = "babel";
})(Printer = exports.Printer || (exports.Printer = {}));
class Config {
    constructor(sourcePaths = [], localPlugins = [], remotePlugins = [], pluginOptions = new Map(), printer = Printer.Recast, extensions = extensions_1.TransformableExtensions, requires = [], transpilePlugins = true, findBabelConfig = false, ignore = ignoreDotfiles, stdio = false, dry = false) {
        this.sourcePaths = sourcePaths;
        this.localPlugins = localPlugins;
        this.remotePlugins = remotePlugins;
        this.pluginOptions = pluginOptions;
        this.printer = printer;
        this.extensions = extensions;
        this.requires = requires;
        this.transpilePlugins = transpilePlugins;
        this.findBabelConfig = findBabelConfig;
        this.ignore = ignore;
        this.stdio = stdio;
        this.dry = dry;
        this.pluginLoader = new PluginLoader_1.default([
            new FileSystemResolver_1.default(),
            new PackageResolver_1.default()
        ]);
        this.remotePluginLoader = new PluginLoader_1.default([
            new AstExplorerResolver_1.default(),
            new NetworkResolver_1.default()
        ]);
    }
    getPlugins() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._pluginCache) {
                let localPlugins = Promise.all(this.localPlugins.map((localPlugin) => __awaiter(this, void 0, void 0, function* () {
                    let pluginExports = yield this.pluginLoader.load(localPlugin);
                    let defaultExport = pluginExports['default'] || pluginExports;
                    return new Plugin(defaultExport, path_1.basename(localPlugin, path_1.extname(localPlugin)));
                })));
                let remotePlugins = Promise.all(this.remotePlugins.map((remotePlugin) => __awaiter(this, void 0, void 0, function* () {
                    let pluginExports = yield this.remotePluginLoader.load(remotePlugin);
                    let defaultExport = pluginExports['default'] || pluginExports;
                    return new Plugin(defaultExport, path_1.basename(remotePlugin, path_1.extname(remotePlugin)));
                })));
                this._pluginCache = [...(yield localPlugins), ...(yield remotePlugins)];
            }
            return this._pluginCache;
        });
    }
    loadRequires() {
        for (let modulePath of this.requires) {
            require(modulePath);
        }
    }
    loadBabelTranspile() {
        if (this.transpilePlugins) {
            transpile_requires_1.enable(this.findBabelConfig);
            source_map_support_1.install();
        }
    }
    unloadBabelTranspile() {
        if (this.transpilePlugins) {
            transpile_requires_1.disable();
        }
    }
    getPlugin(name) {
        return __awaiter(this, void 0, void 0, function* () {
            for (let plugin of yield this.getPlugins()) {
                if (plugin.declaredName === name || plugin.inferredName === name) {
                    return plugin;
                }
            }
            return null;
        });
    }
    getBabelPlugins() {
        return __awaiter(this, void 0, void 0, function* () {
            let result = [AllSyntaxPlugin_1.default];
            switch (this.printer) {
                case Printer.Recast:
                    result.push(RecastPlugin_1.default);
                    break;
                case Printer.Babel:
                    result.push(BabelPrinterPlugin_1.default);
                    break;
                case Printer.Prettier:
                    result.push(PrettierPrinterPlugin_1.default);
                    break;
            }
            for (let plugin of yield this.getPlugins()) {
                let options = (plugin.declaredName && this.pluginOptions.get(plugin.declaredName)) ||
                    this.pluginOptions.get(plugin.inferredName);
                if (options) {
                    result.push([plugin.rawPlugin, options]);
                }
                else {
                    result.push(plugin.rawPlugin);
                }
            }
            return result;
        });
    }
    getBabelPlugin(name) {
        return __awaiter(this, void 0, void 0, function* () {
            let plugin = yield this.getPlugin(name);
            if (!plugin) {
                return null;
            }
            let options = this.pluginOptions.get(name);
            if (options) {
                return [plugin.rawPlugin, options];
            }
            else {
                return plugin.rawPlugin;
            }
        });
    }
}
exports.default = Config;
class ConfigBuilder {
    constructor() {
        this._extensions = new Set(extensions_1.TransformableExtensions);
    }
    sourcePaths(value) {
        this._sourcePaths = value;
        return this;
    }
    addSourcePath(value) {
        if (!this._sourcePaths) {
            this._sourcePaths = [];
        }
        this._sourcePaths.push(value);
        return this;
    }
    addSourcePaths(...values) {
        for (let value of values) {
            this.addSourcePath(value);
        }
        return this;
    }
    localPlugins(value) {
        this._localPlugins = value;
        return this;
    }
    addLocalPlugin(value) {
        if (!this._localPlugins) {
            this._localPlugins = [];
        }
        this._localPlugins.push(value);
        return this;
    }
    remotePlugins(value) {
        this._remotePlugins = value;
        return this;
    }
    addRemotePlugin(value) {
        if (!this._remotePlugins) {
            this._remotePlugins = [];
        }
        this._remotePlugins.push(value);
        return this;
    }
    pluginOptions(value) {
        this._pluginOptions = value;
        return this;
    }
    setOptionsForPlugin(options, plugin) {
        if (!this._pluginOptions) {
            this._pluginOptions = new Map();
        }
        this._pluginOptions.set(plugin, options);
        return this;
    }
    printer(value) {
        this._printer = value;
        return this;
    }
    extensions(value) {
        this._extensions = value;
        return this;
    }
    addExtension(value) {
        if (!this._extensions) {
            this._extensions = new Set();
        }
        this._extensions.add(value);
        return this;
    }
    requires(value) {
        this._requires = value;
        return this;
    }
    addRequire(value) {
        if (!this._requires) {
            this._requires = [];
        }
        this._requires.push(value);
        return this;
    }
    transpilePlugins(value) {
        this._transpilePlugins = value;
        return this;
    }
    findBabelConfig(value) {
        this._findBabelConfig = value;
        return this;
    }
    ignore(value) {
        this._ignore = value;
        return this;
    }
    stdio(value) {
        this._stdio = value;
        return this;
    }
    dry(value) {
        this._dry = value;
        return this;
    }
    build() {
        return new Config(this._sourcePaths, this._localPlugins, this._remotePlugins, this._pluginOptions, this._printer, this._extensions, this._requires, this._transpilePlugins, this._findBabelConfig, this._ignore, this._stdio, this._dry);
    }
}
exports.ConfigBuilder = ConfigBuilder;
//# sourceMappingURL=Config.js.map