"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const globby_1 = require("globby");
const path_1 = require("path");
const resolve_1 = require("resolve");
const Config_1 = require("./Config");
const extensions_1 = require("./extensions");
class Options {
    constructor(args) {
        this.args = args;
    }
    parse() {
        let config = new Config_1.ConfigBuilder();
        for (let i = 0; i < this.args.length; i++) {
            let arg = this.args[i];
            switch (arg) {
                case '-p':
                case '--plugin':
                    i++;
                    config.addLocalPlugin(this.args[i]);
                    break;
                case '--remote-plugin':
                    i++;
                    config.addRemotePlugin(this.args[i]);
                    break;
                case '-o':
                case '--plugin-options':
                    i++;
                    let nameAndOptions = this.args[i].split('=');
                    let name = nameAndOptions[0];
                    let optionsRaw = nameAndOptions[1];
                    if (optionsRaw && optionsRaw[0] === '@') {
                        optionsRaw = fs_1.readFileSync(optionsRaw.slice(1), {
                            encoding: 'utf8'
                        });
                        config.setOptionsForPlugin(JSON.parse(optionsRaw), name);
                    }
                    try {
                        config.setOptionsForPlugin(JSON.parse(optionsRaw), name);
                    }
                    catch (err) {
                        throw new Error(`unable to parse JSON config for ${name}: ${optionsRaw}`);
                    }
                    break;
                case '--printer':
                    i++;
                    let rawPrinter = this.args[i];
                    if (rawPrinter === Config_1.Printer.Babel) {
                        config.printer(Config_1.Printer.Babel);
                    }
                    else if (rawPrinter === Config_1.Printer.Prettier) {
                        config.printer(Config_1.Printer.Prettier);
                    }
                    else if (rawPrinter === Config_1.Printer.Recast) {
                        config.printer(Config_1.Printer.Recast);
                    }
                    else {
                        throw new Error(`unexpected printer value: ${rawPrinter}`);
                    }
                    break;
                case '-r':
                case '--require':
                    i++;
                    config.addRequire(getRequirableModulePath(this.args[i]));
                    break;
                case '--transpile-plugins':
                case '--no-transpile-plugins':
                    config.transpilePlugins(arg === '--transpile-plugins');
                    break;
                case '--find-babel-config':
                case '--no-find-babel-config':
                    config.findBabelConfig(arg === '--find-babel-config');
                    break;
                case '--extensions':
                    i++;
                    config.extensions(new Set(this.args[i]
                        .split(',')
                        .map(ext => (ext[0] === '.' ? ext : `.${ext}`))));
                    break;
                case '--add-extension':
                    i++;
                    config.addExtension(this.args[i]);
                    break;
                case '-s':
                case '--stdio':
                    config.stdio(true);
                    break;
                case '-h':
                case '--help':
                    return { kind: 'help' };
                case '--version':
                    return { kind: 'version' };
                case '-d':
                case '--dry':
                    config.dry(true);
                    break;
                default:
                    if (arg[0] === '-') {
                        throw new Error(`unexpected option: ${arg}`);
                    }
                    else {
                        if (globby_1.hasMagic(arg)) {
                            config.addSourcePaths(...globby_1.sync(arg));
                        }
                        else {
                            config.addSourcePath(arg);
                        }
                    }
                    break;
            }
        }
        return {
            kind: 'run',
            config: config.build()
        };
    }
}
exports.default = Options;
/**
 * Gets a path that can be passed to `require` for a given module path.
 */
function getRequirableModulePath(modulePath) {
    if (fs_1.existsSync(modulePath)) {
        return path_1.resolve(modulePath);
    }
    for (let ext of extensions_1.RequireableExtensions) {
        if (fs_1.existsSync(modulePath + ext)) {
            return path_1.resolve(modulePath + ext);
        }
    }
    return resolve_1.sync(modulePath, { basedir: process.cwd() });
}
//# sourceMappingURL=Options.js.map