"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// tslint:enable no-any
/**
 * Snapshots various global values and allows rolling back to the snapshot.
 *
 * In particular, it:
 * - Restores `require.cache`, typically useful for allowing modules to be
 *   loaded again fresh.
 * - Restores `require.extensions`, typically useful for unloading loader hooks
 *   such as babel-register or ts-node.
 * - Restores event listeners on `process`, typically useful for preventing
 *   hooks (especially 'exit') from being called on un-cached modules.
 */
class ProcessSnapshot {
    constructor(requireImpl = require, processImpl = process, log = () => { }) {
        this.requireImpl = requireImpl;
        this.processImpl = processImpl;
        this.log = log;
        this.cacheEntries = this.snapshotRequireCache();
        this.extensions = this.snapshotRequireExtensions();
        this.processEvents = this.snapshotProcessEvents();
    }
    /**
     * Restores the snapshot that was captured on construction.
     */
    restore() {
        this.restoreRequireCache();
        this.restoreExtensions();
        this.restoreProcessEvents();
    }
    /**
     * Helper to diff two maps used as snapshots.
     */
    static diffMaps(before, after) {
        let added = new Map();
        let modified = new Map();
        let deleted = new Map();
        for (let [beforeKey, beforeValue] of before) {
            let afterValue = after.get(beforeKey);
            if (after.has(beforeKey)) {
                if (afterValue !== beforeValue) {
                    modified.set(beforeKey, [beforeValue, afterValue]);
                }
            }
            else {
                deleted.set(beforeKey, beforeValue);
            }
        }
        for (let [afterKey, afterValue] of after) {
            if (!before.has(afterKey)) {
                added.set(afterKey, afterValue);
            }
        }
        return { added, modified, deleted };
    }
    /**
     * Restores `require.cache` in place.
     */
    restoreRequireCache() {
        let { cache } = this.requireImpl;
        let { added, modified, deleted } = ProcessSnapshot.diffMaps(this.cacheEntries, this.snapshotRequireCache());
        for (let key of added.keys()) {
            this.log(`removing ${key} from require cache`);
            delete cache[key];
        }
        for (let [key, [original]] of modified) {
            this.log(`restoring replaced ${key} in require cache`);
            cache[key] = original;
        }
        for (let [key, original] of deleted) {
            this.log(`restoring deleted ${key} to require cache`);
            cache[key] = original;
        }
    }
    /**
     * Restores `require.extensions` in place.
     */
    restoreExtensions() {
        let { extensions } = this.requireImpl;
        let { added, modified, deleted } = ProcessSnapshot.diffMaps(this.extensions, this.snapshotRequireExtensions());
        for (let key of added.keys()) {
            this.log(`removing ${key} from require extensions`);
            delete extensions[key];
        }
        for (let [key, [original]] of modified) {
            this.log(`restoring replaced ${key} in require extensions`);
            extensions[key] = original;
        }
        for (let [key, original] of deleted) {
            this.log(`restoring deleted ${key} to require extensions`);
            extensions[key] = original;
        }
    }
    /**
     * Restores `process` events using the `EventEmitter` API.
     */
    restoreProcessEvents() {
        let process = this.processImpl;
        let { added, modified, deleted } = ProcessSnapshot.diffMaps(this.processEvents, this.snapshotProcessEvents());
        for (let event of added.keys()) {
            this.log(`removing all '${event.toString()}' event listeners`);
            process.removeAllListeners(event);
        }
        for (let [event, [original, updated]] of modified) {
            for (let originalEntry of original) {
                if (!updated.includes(originalEntry)) {
                    this.log(`restoring removed '${event.toString()}' event listener`);
                    process.addListener(event, originalEntry); // tslint:disable-line no-any
                }
            }
            for (let updatedEntry of updated) {
                if (!original.includes(updatedEntry)) {
                    this.log(`removing added '${event.toString()}' event listener`);
                    process.removeListener(event, updatedEntry);
                }
            }
        }
        for (let [event, callbacks] of deleted) {
            for (let callback of callbacks) {
                this.log(`restoring removed '${event.toString()}' event listener`);
                process.addListener(event, callback); // tslint:disable-line no-any
            }
        }
    }
    /**
     * Snapshots the current state of `require.cache`.
     */
    snapshotRequireCache() {
        let result = new Map();
        let { cache } = this.requireImpl;
        for (let path in cache) {
            if (Object.prototype.hasOwnProperty.call(cache, path)) {
                result.set(path, cache[path]);
            }
        }
        return result;
    }
    /**
     * Snapshots the current state of `require.extensions`.
     */
    snapshotRequireExtensions() {
        let result = new Map();
        let { extensions } = this.requireImpl;
        for (let key in extensions) {
            if (Object.prototype.hasOwnProperty.call(extensions, key)) {
                result.set(key, extensions[key]);
            }
        }
        return result;
    }
    /**
     * Snapshots the current state of `process` events.
     */
    snapshotProcessEvents() {
        let result = new Map();
        let events = this.processImpl.eventNames();
        for (let name of events) {
            // tslint:disable no-any
            result.set(name, this.processImpl.listeners(name));
            // tslint:enable no-any
        }
        return result;
    }
}
exports.default = ProcessSnapshot;
//# sourceMappingURL=ProcessSnapshot.js.map