"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const realFs = require("fs");
const getStream = require("get-stream");
const path_1 = require("path");
const CLIEngine_1 = require("./CLIEngine");
const Config_1 = require("./Config");
const Options_1 = require("./Options");
const TransformRunner_1 = require("./TransformRunner");
// Polyfill `Symbol.asyncIterator` so `for await` will work.
if (!Symbol.asyncIterator) {
    Symbol['asyncIterator'] =
        Symbol.asyncIterator || Symbol.for('Symbol.asyncIterator');
}
function optionAnnotation(value) {
    if (Array.isArray(value) || value instanceof Map) {
        return ' (allows multiple)';
    }
    else if (typeof value === 'boolean') {
        return ` (default: ${value ? 'on' : 'off'})`;
    }
    else if (typeof value === 'string') {
        return ` (default: ${value})`;
    }
    else {
        return '';
    }
}
function printHelp(argv, out) {
    let $0 = path_1.basename(argv[1]);
    let defaults = new Config_1.default();
    out.write(`
${$0} [OPTIONS] [PATH … | --stdio]

OPTIONS
  -p, --plugin PLUGIN               Transform sources with PLUGIN${optionAnnotation(defaults.localPlugins)}.
      --remote-plugin URL           Fetch a plugin from URL${optionAnnotation(defaults.remotePlugins)}.
  -o, --plugin-options PLUGIN=OPTS  JSON-encoded OPTS for PLUGIN${optionAnnotation(defaults.pluginOptions)}.
  -r, --require PATH                Require PATH before transform${optionAnnotation(defaults.requires)}.
      --add-extension EXT           Add an extension to the list of supported extensions.
      --extensions EXTS             Comma-separated extensions to process (default: "${Array.from(defaults.extensions).join(',')}").
      --[no-]transpile-plugins      Transpile plugins to enable future syntax${optionAnnotation(defaults.transpilePlugins)}.
      --[no-]find-babel-config      Run plugins through babel plugins/presets specified in local
                                    .babelrc file instead of babel-preset-env${optionAnnotation(defaults.findBabelConfig)}.
      --printer PRINTER             Specify which printer to use${optionAnnotation(defaults.printer)}.
  -s, --stdio                       Read source from stdin and print to stdout${optionAnnotation(defaults.stdio)}.
  -d, --dry                         Run plugins without modifying files on disk${optionAnnotation(defaults.dry)}.
      --version                     Print the version of ${$0}.
  -h, --help                        Show this help message.

  NOTE: \`--remote-plugin\` should only be used as a convenience to load code that you or someone
        you trust wrote. It will run with your full user privileges, so please exercise caution!

EXAMPLES
  # Run with a relative plugin on all files in \`src/\`.
  $ ${$0} -p ./typecheck.js src/

  # Run with a remote plugin from astexplorer.net on all files in \`src/\`.
  $ ${$0} --remote-plugin 'https://astexplorer.net/#/gist/688274…' src/

  # Run with multiple plugins.
  $ ${$0} -p ./a.js -p ./b.js some-file.js

  # Transform TypeScript sources.
  # ${$0} -p ./a.js my-typescript-file.ts a-component.tsx

  # Run with a plugin in \`node_modules\` on stdin.
  $ ${$0} -s -p babel-plugin-typecheck <<EOS
  function add(a: number, b: number): number {
    return a + b;
  }
  EOS

  # Reprint modified files with prettier.
  $ ${$0} --printer prettier -p some-plugin src/

  # Pass options to a plugin.
  $ ${$0} -p ./a.js -o a='{"foo":true}' src/

  # Pass options from a config file to a plugin.
  $ ${$0} -p ./a.js -o a=@opts.json src/

  # Run with a plugin written in TypeScript.
  $ ${$0} -p ./some-plugin.ts src/
  `.trim());
    out.write('\n');
}
function printVersion(argv, out) {
    out.write(require('../package.json').version);
    out.write('\n');
}
function run(argv, stdin, stdout, stderr, fs = realFs) {
    return __awaiter(this, void 0, void 0, function* () {
        let command;
        try {
            command = new Options_1.default(argv.slice(2)).parse();
        }
        catch (error) {
            stderr.write(`ERROR: ${error.message}\n`);
            printHelp(argv, stderr);
            return 1;
        }
        if (command.kind === 'help') {
            printHelp(argv, stdout);
            return 0;
        }
        if (command.kind === 'version') {
            printVersion(argv, stdout);
            return 0;
        }
        let config = command.config;
        let dim = stdout.isTTY ? '\x1b[2m' : '';
        let reset = stdout.isTTY ? '\x1b[0m' : '';
        function onTransform(result) {
            if (result.kind === TransformRunner_1.SourceTransformResultKind.Transformed) {
                if (!config.stdio) {
                    if (result.output === result.source.content) {
                        stdout.write(`${dim}${result.source.path}${reset}\n`);
                    }
                    else {
                        stdout.write(`${result.source.path}\n`);
                    }
                }
            }
            else if (result.error) {
                if (!config.stdio) {
                    stderr.write(`Encountered an error while processing ${result.source.path}:\n`);
                }
                stderr.write(`${result.error.stack}\n`);
            }
        }
        let { stats } = yield new CLIEngine_1.default(config, onTransform, () => __awaiter(this, void 0, void 0, function* () { return yield getStream(stdin); }), (data) => stdout.write(data), fs).run();
        if (!config.stdio) {
            if (config.dry) {
                stdout.write('DRY RUN: no files affected\n');
            }
            stdout.write(`${stats.total} file(s), ${stats.modified} modified, ${stats.errors} errors\n`);
        }
        // exit status is number of errors up to byte max value
        return Math.min(stats.errors, 255);
    });
}
exports.default = run;
//# sourceMappingURL=index.js.map